@@ -1,6 +1,6 @@
 /*  filter.c -- filter expressions.
 
-    Copyright (C) 2013-2014 Genome Research Ltd.
+    Copyright (C) 2013-2015 Genome Research Ltd.
 
     Author: Petr Danecek <pd3@sanger.ac.uk>
 
@@ -41,7 +41,7 @@ typedef struct _token_t
     char *key;          // set only for string constants, otherwise NULL
     char *tag;          // for debugging and printout only, VCF tag name
     float threshold;    // filtering threshold
-    int hdr_id;         // BCF header lookup ID
+    int hdr_id, type;   // BCF header lookup ID and one of BCF_HT_* types
     int idx;            // 0-based index to VCF vectors, -1: not a vector, -2: any field ([*])
     void (*setter)(filter_t *, bcf1_t *, struct _token_t *);
     int (*comparator)(struct _token_t *, struct _token_t *, int op_type, bcf1_t *);
@@ -51,7 +51,7 @@ typedef struct _token_t
     // modified on filter evaluation at each VCF line
     float *values;      // In case str_value is set, values[0] is one sample's string length
     char *str_value;    //  and values[0]*nsamples gives the total length;
-    int is_str;
+    int is_str, is_missing; // is_missing is set only for constants, variables are controled via nvalues
     int pass_site;          // -1 not applicable, 0 fails, >0 pass
     uint8_t *pass_samples;  // status of individual samples
     int nsamples;           // number of samples
@@ -251,15 +251,25 @@ static void filters_set_info(filter_t *flt, bcf1_t *line, token_t *tok)
     }
     else if ( line->d.info[i].type==BCF_BT_FLOAT )
     {
-        tok->values[0] = line->d.info[i].v1.f;
+        if ( bcf_float_is_missing(line->d.info[i].v1.f) ) tok->nvalues = 0;
+        else
+        {
+            tok->values[0] = line->d.info[i].v1.f;
+            tok->nvalues   = 1;
+        }
         tok->str_value = NULL;
-        tok->nvalues   = 1;
     }
     else
     {
-        tok->values[0] = line->d.info[i].v1.i;
+        if ( line->d.info[i].type==BCF_BT_INT8 && line->d.info[i].v1.i==bcf_int8_missing ) tok->nvalues = 0;
+        else if ( line->d.info[i].type==BCF_BT_INT16 && line->d.info[i].v1.i==bcf_int16_missing ) tok->nvalues = 0;
+        else if ( line->d.info[i].type==BCF_BT_INT32 && line->d.info[i].v1.i==bcf_int32_missing ) tok->nvalues = 0;
+        else
+        {
+            tok->values[0] = line->d.info[i].v1.i;
+            tok->nvalues   = 1;
+        }
         tok->str_value = NULL;
-        tok->nvalues   = 1;
     }
 }
 static int filters_cmp_filter(token_t *atok, token_t *btok, int op_type, bcf1_t *line)
@@ -354,6 +364,12 @@ static int bcf_get_info_value(bcf1_t *line, int info_id, int ivec, void *value)
     return -1;  // this shouldn't happen
 }
 
+static void filters_set_pos(filter_t *flt, bcf1_t *line, token_t *tok)
+{
+    tok->values[0] = line->pos+1;
+    tok->nvalues = 1;
+}
+
 static void filters_set_info_int(filter_t *flt, bcf1_t *line, token_t *tok)
 {
     if ( tok->idx==-2 )
@@ -490,6 +506,7 @@ static void filters_set_format_float(filter_t *flt, bcf1_t *line, token_t *tok)
             tok->nsamples = tok->nvalues = nsmpl;
         }
     }
+    tok->nsamples = tok->nvalues;
 }
 static void filters_set_format_string(filter_t *flt, bcf1_t *line, token_t *tok)
 {
@@ -551,12 +568,39 @@ static void filters_set_genotype_string(filter_t *flt, bcf1_t *line, token_t *to
         return;
     }
     int i, blen = 3, nsmpl = bcf_hdr_nsamples(flt->hdr);
-    kstring_t str; str.s = tok->str_value; str.m = tok->values[0] * nsmpl; str.l = 0;
+    kstring_t str;
+
+gt_length_too_big:
+    str.s = tok->str_value; str.m = tok->values[0] * nsmpl; str.l = 0;
     for (i=0; i<nsmpl; i++)
     {
         int plen = str.l;
-        bcf_format_gt(fmt, i, &str);
-        assert( str.l - plen <= blen ); // increase blen if this fails
+
+        #define BRANCH(type_t) { \
+            type_t *ptr = (type_t*) (fmt->p + i*fmt->size); \
+            if ( !(ptr[0]>>1) ) kputc('.',&str); \
+        }
+        switch (fmt->type) {
+            case BCF_BT_INT8:  BRANCH(int8_t); break;
+            case BCF_BT_INT16: BRANCH(int16_t); break;
+            case BCF_BT_INT32: BRANCH(int32_t); break;
+            default: fprintf(stderr,"FIXME: type %d in bcf_format_gt?\n", fmt->type); abort(); break;
+        }
+        #undef BRANCH
+
+        if ( plen==str.l )
+        {
+            bcf_format_gt(fmt, i, &str);
+            if ( str.l - plen > blen )
+            {
+                // too many alternate alleles or ploidy is too large, the genotype does not fit
+                // three characters ("0/0" vs "10/10").
+                tok->str_value = str.s;
+                blen *= 2;
+                goto gt_length_too_big;
+            }
+        }
+
         plen = str.l - plen;
         while ( plen<blen )
         {
@@ -587,7 +631,7 @@ static void filters_set_alt_string(filter_t *flt, bcf1_t *line, token_t *tok)
         else
             kputc('.', &str);
     }
-    else
+    else if ( line->n_allele>1 )
     {
         kputs(line->d.allele[1], &str);
         int i;
@@ -597,6 +641,8 @@ static void filters_set_alt_string(filter_t *flt, bcf1_t *line, token_t *tok)
             kputs(line->d.allele[i], &str);
         }
     }
+    else if ( line->n_allele==1 )
+        kputc('.', &str);
     tok->nvalues = str.l;
     tok->values[0] = str.m;
     tok->str_value = str.s;
@@ -625,7 +671,12 @@ static void filters_set_ac(filter_t *flt, bcf1_t *line, token_t *tok)
     if ( tok->idx>=0 )
     {
         tok->nvalues = 1;
-        tok->values[0] = flt->tmpi[tok->idx+1];
+        tok->values[0] = tok->idx+1<line->n_allele ? flt->tmpi[tok->idx+1] : 0;
+    }
+    else if ( line->n_allele==1 )   // no ALT
+    {
+        tok->nvalues = 1;
+        tok->values[0] = 0;
     }
     else
     {
@@ -800,7 +851,7 @@ static void set_strlen(filter_t *flt, bcf1_t *line, token_t *tok)
     if ( !has_values ) { (atok)->nvalues = 0; (atok)->nsamples = 0; } \
 }
 
-static int vector_logic_and(token_t *atok, token_t *btok)
+static int vector_logic_and(token_t *atok, token_t *btok, int and_type)
 {
     // We are comparing either two scalars (result of INFO tag vs a threshold), two vectors (two FORMAT fields),
     // or a vector and a scalar (FORMAT field vs threshold)
@@ -813,10 +864,29 @@ static int vector_logic_and(token_t *atok, token_t *btok)
     if ( !atok->nsamples && !btok->nsamples ) return atok->pass_site && btok->pass_site;
     if ( atok->nsamples && btok->nsamples )
     {
-        for (i=0; i<atok->nsamples; i++)
+        if ( and_type==TOK_AND )
         {
-            atok->pass_samples[i] = atok->pass_samples[i] && btok->pass_samples[i];
-            if ( !pass_site && atok->pass_samples[i] ) pass_site = 1;
+            // perform AND within a sample
+            for (i=0; i<atok->nsamples; i++)
+            {
+                atok->pass_samples[i] = atok->pass_samples[i] && btok->pass_samples[i];
+                if ( !pass_site && atok->pass_samples[i] ) pass_site = 1;
+            }
+        }
+        else
+        {
+            // perform AND across samples
+            int pass_a = 0, pass_b = 0;
+            for (i=0; i<atok->nsamples; i++)
+            {
+                if ( atok->pass_samples[i] ) pass_a = 1;
+                atok->pass_samples[i] = atok->pass_samples[i] && btok->pass_samples[i];
+            }
+            for (i=0; i<btok->nsamples; i++)
+            {
+                if ( btok->pass_samples[i] ) { pass_b = 1; break; }
+            }
+            pass_site = pass_a && pass_b;
         }
         return pass_site;
     }
@@ -903,6 +973,13 @@ static int vector_logic_or(token_t *atok, token_t *btok, int or_type)
     return pass_site;
 }
 
+#define CMP_MISSING(atok,btok,CMP_OP,ret) \
+{ \
+    if ( (atok)->nsamples || (btok)->nsamples ) error("todo: Querying of missing values in FORMAT\n"); \
+    token_t *tok = (atok)->is_missing ? (btok) : (atok); \
+    (ret) = ( tok->nvalues CMP_OP 1 ) ? 0 : 1; \
+}
+
 #define CMP_VECTORS(atok,btok,CMP_OP,ret) \
 { \
     int i, j, has_values = 0, pass_site = 0; \
@@ -964,11 +1041,7 @@ static int vector_logic_or(token_t *atok, token_t *btok, int or_type)
         } \
         else \
         { \
-            if ( bcf_float_is_missing((atok)->values[0]) || bcf_float_is_missing((btok)->values[0]) ) \
-            { \
-                (atok)->nvalues = 0; (atok)->nsamples = 0; (ret) = 0; \
-            } \
-            else if ( (atok)->values[0] CMP_OP (btok)->values[0] ) { pass_site = 1; } \
+            if ( (atok)->values[0] CMP_OP (btok)->values[0] ) { pass_site = 1; } \
         } \
         /*fprintf(stderr,"pass=%d\n", pass_site);*/ \
         (ret) = pass_site; \
@@ -1076,6 +1149,7 @@ static int filters_init1(filter_t *filter, char *str, int len, token_t *tok)
         tok->key[len-2] = 0;
         tok->is_str = 1;
         tok->nvalues = len-2;
+        if ( !strcmp(".",tok->key) ) tok->is_missing = 1;
         return 0;
     }
 
@@ -1138,6 +1212,12 @@ static int filters_init1(filter_t *filter, char *str, int len, token_t *tok)
             tok->tag = strdup("ID");
             return 0;
         }
+        else if ( !strncasecmp(str,"POS",len) )
+        {
+            tok->setter = &filters_set_pos;
+            tok->tag = strdup("POS");
+            return 0;
+        }
         else if ( !strncasecmp(str,"REF",len) )
         {
             tok->setter = &filters_set_ref_string;
@@ -1197,6 +1277,7 @@ static int filters_init1(filter_t *filter, char *str, int len, token_t *tok)
         }
         if ( is_fmt==-1 ) is_fmt = 0;
     }
+    tok->type = is_fmt ? BCF_HL_FMT : BCF_HL_INFO;
     if ( is_fmt ) filter->max_unpack |= BCF_UN_FMT;
     if ( tok->hdr_id>=0 )
     {
@@ -1228,25 +1309,25 @@ static int filters_init1(filter_t *filter, char *str, int len, token_t *tok)
             {
                 if ( bcf_hdr_id2type(filter->hdr,BCF_HL_INFO,tok->hdr_id) == BCF_HT_STR ) tok->is_str = 1;
                 if ( bcf_hdr_id2number(filter->hdr,BCF_HL_INFO,tok->hdr_id)==1 )
-                tok->setter = filters_set_info;
-            else
-            {
-                switch ( bcf_hdr_id2type(filter->hdr,BCF_HL_INFO,tok->hdr_id) )
+                    tok->setter = filters_set_info;
+                else
                 {
-                    case BCF_HT_INT:  tok->setter = &filters_set_info_int; break;
-                    case BCF_HT_REAL: tok->setter = &filters_set_info_float; break;
-                    case BCF_HT_STR:  tok->setter = &filters_set_info_string; tok->is_str = 1; break;
-                    default: error("[%s:%d %s] FIXME\n", __FILE__,__LINE__,__FUNCTION__);
+                    switch ( bcf_hdr_id2type(filter->hdr,BCF_HL_INFO,tok->hdr_id) )
+                    {
+                        case BCF_HT_INT:  tok->setter = &filters_set_info_int; break;
+                        case BCF_HT_REAL: tok->setter = &filters_set_info_float; break;
+                        case BCF_HT_STR:  tok->setter = &filters_set_info_string; tok->is_str = 1; break;
+                        default: error("[%s:%d %s] FIXME\n", __FILE__,__LINE__,__FUNCTION__);
+                    }
+                    if(!is_array) tok->idx = -2;
                 }
-                    //tok->idx = -2;
-            }
             }
             filter->max_unpack |= BCF_UN_INFO;
-            }
-            tok->tag = strdup(tmp.s);
-            if ( tmp.s ) free(tmp.s);
-            return 0;
         }
+        tok->tag = strdup(tmp.s);
+        if ( tmp.s ) free(tmp.s);
+        return 0;
+    }
     else if ( !strcasecmp(tmp.s,"ALT") )
     {
         tok->setter = &filters_set_alt_string;
@@ -1420,10 +1501,23 @@ filter_t *filter_init(bcf_hdr_t *hdr, const char *str)
     // list of operators and convert the strings (e.g. "PASS") to BCF ids. The string value token must be
     // just before or after the FILTER token and they must be followed with a comparison operator.
     // At this point we also initialize regex expressions which, in RPN, must preceed the LIKE/NLIKE operator.
+    // Additionally, treat "." as missing value rather than a string in numeric equalities.
     // This code is fragile: improve me.
     int i;
     for (i=0; i<nout; i++)
     {
+        if ( out[i].tok_type==TOK_EQ || out[i].tok_type==TOK_NE )
+        {
+            // Look for j="." and k numeric type
+            int j = i-1, k = i-2;
+            if ( !out[j].is_str ) { k = i-1, j = i-2; }
+            if ( out[k].hdr_id>0 && out[j].is_str && !strcmp(".",out[j].key) )
+            {
+                int type = bcf_hdr_id2type(filter->hdr,out[k].type,out[k].hdr_id);
+                if ( type==BCF_HT_INT ) { out[j].is_str = 0; out[j].is_missing = 1; bcf_float_set_missing(out[j].values[0]); }
+                if ( type==BCF_HT_REAL ) { out[j].is_str = 0; out[j].is_missing = 1; bcf_float_set_missing(out[j].values[0]); }
+            }
+        }
         if ( out[i].tok_type==TOK_LIKE || out[i].tok_type==TOK_NLIKE )
         {
             int j = i-1;
@@ -1455,7 +1549,11 @@ filter_t *filter_init(bcf_hdr_t *hdr, const char *str)
         {
             if ( i+1==nout ) error("Could not parse the expression: %s\n", filter->str);
             int j = i+1;
-            if ( out[j].tok_type==TOK_EQ || out[j].tok_type==TOK_NE || out[j].tok_type==TOK_LIKE ) j = i - 1;
+            if ( out[j].tok_type==TOK_EQ || out[j].tok_type==TOK_NE ) j = i - 1;    // the expression has "value"=FILTER rather than FILTER="value"
+            if ( out[j].tok_type==TOK_LIKE ) out[j].tok_type = TOK_EQ;              // for FILTER, ~ and !~ work the same way as = and !=
+            if ( out[j].tok_type==TOK_NLIKE ) out[j].tok_type = TOK_NE;
+            if ( out[j+1].tok_type==TOK_LIKE ) out[j+1].tok_type = TOK_EQ;
+            if ( out[j+1].tok_type==TOK_NLIKE ) out[j+1].tok_type = TOK_NE;
             if ( out[j].tok_type!=TOK_VAL || !out[j].key )
                 error("[%s:%d %s] Could not parse the expression, an unquoted string value perhaps? %s\n", __FILE__,__LINE__,__FUNCTION__, filter->str);
             if ( strcmp(".",out[j].key) )
@@ -1575,7 +1673,7 @@ int filter_test(filter_t *filter, bcf1_t *line, const uint8_t **samples)
         {
             if ( filter->flt_stack[nstack-1]->pass_site<0 || filter->flt_stack[nstack-2]->pass_site<0 )
                 error("Error occurred while processing the filter \"%s\" (%d %d AND)\n", filter->str,filter->flt_stack[nstack-2]->pass_site,filter->flt_stack[nstack-1]->pass_site);
-            filter->flt_stack[nstack-2]->pass_site = vector_logic_and(filter->flt_stack[nstack-2],filter->flt_stack[nstack-1]);
+            filter->flt_stack[nstack-2]->pass_site = vector_logic_and(filter->flt_stack[nstack-2],filter->flt_stack[nstack-1], filter->filters[i].tok_type);
             nstack--;
             continue;
         }
@@ -1608,7 +1706,16 @@ int filter_test(filter_t *filter, bcf1_t *line, const uint8_t **samples)
         int is_true = 0;
         if ( !filter->flt_stack[nstack-1]->nvalues || !filter->flt_stack[nstack-2]->nvalues )
         {
-            filter->flt_stack[nstack-2]->nvalues = filter->flt_stack[nstack-2]->nsamples = 0;
+            int skip = 0;
+            if ( !filter->flt_stack[nstack-2]->is_missing && !filter->flt_stack[nstack-1]->is_missing ) skip = 1;
+            if ( filter->filters[i].tok_type != TOK_EQ  && filter->filters[i].tok_type != TOK_NE ) skip = 1;
+
+            if ( skip ) 
+                filter->flt_stack[nstack-2]->nvalues = filter->flt_stack[nstack-2]->nsamples = 0;
+            else if ( filter->filters[i].tok_type == TOK_EQ )
+                CMP_MISSING(filter->flt_stack[nstack-2],filter->flt_stack[nstack-1],==,is_true)
+            else if ( filter->filters[i].tok_type == TOK_NE )
+                CMP_MISSING(filter->flt_stack[nstack-2],filter->flt_stack[nstack-1],!=,is_true)
         }
         else if ( filter->filters[i].tok_type == TOK_EQ )
         {
@@ -1675,3 +1782,7 @@ int filter_test(filter_t *filter, bcf1_t *line, const uint8_t **samples)
     return filter->flt_stack[0]->pass_site;
 }
 
+int filter_max_unpack(filter_t *flt)
+{
+    return flt->max_unpack;
+}
