@@ -1,6 +1,6 @@
 /*  vcfcall.c -- SNP/indel variant calling from VCF/BCF.
 
-    Copyright (C) 2013-2014 Genome Research Ltd.
+    Copyright (C) 2013-2016 Genome Research Ltd.
 
     Author: Petr Danecek <pd3@sanger.ac.uk>
 
@@ -34,10 +34,13 @@ THE SOFTWARE.  */
 #include <stdarg.h>
 #include <htslib/kfunc.h>
 #include <htslib/synced_bcf_reader.h>
+#include <htslib/khash_str2int.h>
 #include <ctype.h>
 #include "bcftools.h"
 #include "call.h"
 #include "prob1.h"
+#include "ploidy.h"
+#include "gvcf.h"
 
 void error(const char *format, ...);
 
@@ -49,7 +52,7 @@ void error(const char *format, ...);
 #define CF_NO_GENO      1
 #define CF_INS_MISSED   (1<<1)
 #define CF_CCALL        (1<<2)
-#define CF_GVCF         (1<<3)
+//                      (1<<3)
 //                      (1<<4)
 //                      (1<<5)
 #define CF_ACGT_ONLY    (1<<6)
@@ -65,17 +68,23 @@ void error(const char *format, ...);
 typedef struct
 {
     int flag;   // combination of CF_* flags above
-    int output_type;
+    int output_type, n_threads, record_cmd_line;
     htsFile *bcf_in, *out_fh;
     char *bcf_fname, *output_fname;
     char **samples;             // for subsampling and ploidy
-    int nsamples, *samples_map;
+    int nsamples, *samples_map; // mapping from output sample names to original VCF
     char *regions, *targets;    // regions to process
     int regions_is_file, targets_is_file;
 
+    char *samples_fname;
+    int samples_is_file;
+    int *sample2sex;    // mapping for ploidy. If negative, interpreted as -1*ploidy
+    int *sex2ploidy, *sex2ploidy_prev, nsex;
+    ploidy_t *ploidy;
+    gvcf_t *gvcf;
+
     bcf1_t *missed_line;
     call_t aux;     // parameters and temporary data
-    gvcf_t gvcf;
 
     int argc;
     char **argv;
@@ -92,43 +101,101 @@ typedef struct
 }
 args_t;
 
-static family_t *get_family(family_t *fam, int nfam, char *name)
+static char **add_sample(void *name2idx, char **lines, int *nlines, int *mlines, char *name, char sex, int *ith)
 {
-    int i;
-    for (i=0; i<nfam; i++)
-    {
-        if ( !strcmp(fam[i].name, name) ) return &fam[i];
-    }
-    return NULL;
-}
+    int ret = khash_str2int_get(name2idx, name, ith);
+    if ( ret==0 ) return lines;
 
-static char **add_sample(char **sam, int *n, int *m, char *name, int ploidy, int *ith)
-{
-    int i;
-    for (i=0; i<*n; i++)
-    {
-        if ( !strcmp(sam[i], name) )
-        {
-            *ith = i;
-            return sam;
-        }
-    }
-    hts_expand(char*,(*n+1),*m,sam);
+    hts_expand(char*,(*nlines+1),*mlines,lines);
     int len = strlen(name);
-    sam[*n] = (char*) malloc(len+2);
-    memcpy(sam[*n],name,len+1);
-    sam[*n][len+1] = ploidy;
-    *ith = *n;
-    (*n)++;
-    return sam;
+    lines[*nlines] = (char*) malloc(len+3);
+    memcpy(lines[*nlines],name,len);
+    lines[*nlines][len]   = ' ';
+    lines[*nlines][len+1] = sex;
+    lines[*nlines][len+2] = 0;
+    *ith = *nlines;
+    (*nlines)++;
+    khash_str2int_set(name2idx, strdup(name), *ith);
+    return lines;
 }
 
-static char **parse_ped_samples(call_t *call, char **vals, int _n)
+typedef struct
 {
-    int i, j, max = 0, n = 0;
-    kstring_t str = {0,0,0};
-    char **sam = NULL;
-    for (i=0; i<_n; i++)
+    const char *alias, *about, *ploidy;
+}
+ploidy_predef_t;
+
+static ploidy_predef_t ploidy_predefs[] =
+{
+    { .alias  = "GRCh37",
+      .about  = "Human Genome reference assembly GRCh37 / hg19",
+      .ploidy =
+          "X 1 60000 M 1\n"
+          "X 2699521 154931043 M 1\n"
+          "Y 1 59373566 M 1\n"
+          "Y 1 59373566 F 0\n"
+          "MT 1 16569 M 1\n"
+          "MT 1 16569 F 1\n"
+          "chrX 1 60000 M 1\n"
+          "chrX 2699521 154931043 M 1\n"
+          "chrY 1 59373566 M 1\n"
+          "chrY 1 59373566 F 0\n"
+          "chrM 1 16569 M 1\n"
+          "chrM 1 16569 F 1\n"
+          "*  * *     M 2\n"
+          "*  * *     F 2\n"
+    },
+    { .alias  = "GRCh38",
+      .about  = "Human Genome reference assembly GRCh38 / hg38, plain chromosome naming (1,2,3,..)",
+      .ploidy =
+          "X 1 9999 M 1\n"
+          "X 2781480 155701381 M 1\n"
+          "Y 1 57227415 M 1\n"
+          "Y 1 57227415 F 0\n"
+          "MT 1 16569 M 1\n"
+          "MT 1 16569 F 1\n"
+          "chrX 1 9999 M 1\n"
+          "chrX 2781480 155701381 M 1\n"
+          "chrY 1 57227415 M 1\n"
+          "chrY 1 57227415 F 0\n"
+          "chrM 1 16569 M 1\n"
+          "chrM 1 16569 F 1\n"
+          "*  * *     M 2\n"
+          "*  * *     F 2\n"
+    },
+    { .alias  = "X",
+      .about  = "Treat male samples as haploid and female as diploid regardless of the chromosome name",
+      .ploidy =
+          "*  * *     M 1\n"
+          "*  * *     F 2\n"
+    },
+    { .alias  = "Y",
+      .about  = "Treat male samples as haploid and female as no-copy, regardless of the chromosome name",
+      .ploidy =
+          "*  * *     M 1\n"
+          "*  * *     F 0\n"
+    },
+    { .alias  = "1",
+      .about  = "Treat all samples as haploid",
+      .ploidy =
+          "*  * *     * 1\n"
+    },
+    {
+        .alias  = NULL,
+        .about  = NULL,
+        .ploidy = NULL,
+    }
+};
+
+// only 5 columns are required and the first is ignored:
+//  ignored,sample,father(or 0),mother(or 0),sex(1=M,2=F)
+static char **parse_ped_samples(call_t *call, char **vals, int nvals, int *nsmpl)
+{
+    int i, j, mlines = 0, nlines = 0;
+    kstring_t str = {0,0,0}, fam_str = {0,0,0};
+    void *name2idx = khash_str2int_init();
+    char **lines = NULL;
+    for (i=0; i<nvals; i++)
     {
         str.l = 0;
         kputs(vals[i], &str);
@@ -149,49 +216,38 @@ static char **parse_ped_samples(call_t *call, char **vals, int _n)
         }
         if ( j!=5 ) break;
 
-        family_t *fam = get_family(call->fams, call->nfams, str.s);
-        if ( !fam )
+        char sex = col_ends[3][1]=='1' ? 'M' : 'F';
+        lines = add_sample(name2idx, lines, &nlines, &mlines, col_ends[0]+1, sex, &j);
+        if ( strcmp(col_ends[1]+1,"0") && strcmp(col_ends[2]+1,"0") )   // father and mother
         {
             call->nfams++;
             hts_expand(family_t, call->nfams, call->mfams, call->fams);
-            fam = &call->fams[call->nfams-1];
-            fam->name = strdup(str.s);
-            for (j=0; j<3; j++) fam->sample[j] = -1;
-        }
+            family_t *fam = &call->fams[call->nfams-1];
+            fam_str.l = 0;
+            ksprintf(&fam_str,"father=%s, mother=%s, child=%s", col_ends[1]+1,col_ends[2]+1,col_ends[0]+1);
+            fam->name = strdup(fam_str.s);
 
-        int ploidy = 2;
-        if ( call->flag & (CALL_CHR_X|CALL_CHR_Y) )
-        {
-            if ( col_ends[3][1]=='1' ) ploidy = 1; // male: one chrX and one chrY copy
-            else
-                ploidy = call->flag & CALL_CHR_X ? 2 : 0; // female: two chrX copies, no chrY
-        }
-        sam = add_sample(sam, &n, &max, col_ends[0]+1, ploidy, &j);
-        if ( strcmp(col_ends[1]+1,"0") )    // father
-        {
-            if ( fam->sample[CHILD]>=0 ) error("Multiple childs in %s [%s,%s]\n", str.s, sam[j],sam[fam->sample[CHILD]]);
-            fam->sample[CHILD] = j;
-            if ( fam->sample[FATHER]>=0 ) error("Two fathers in %s?\n", str.s);
-            sam = add_sample(sam, &n, &max, col_ends[1]+1, call->flag & (CALL_CHR_X|CALL_CHR_Y) ? 1 : 2, &fam->sample[FATHER]);
-        }
-        if ( strcmp(col_ends[2]+1,"0") )    // mother
-        {
-            if ( fam->sample[MOTHER]>=0 ) error("Two mothers in %s?\n", str.s);
-            sam = add_sample(sam, &n, &max, col_ends[2]+1, call->flag & CALL_CHR_Y ? 0 : 2, &fam->sample[MOTHER]);
+            if ( !khash_str2int_has_key(name2idx, col_ends[1]+1) )
+                lines = add_sample(name2idx, lines, &nlines, &mlines, col_ends[1]+1, 'M', &fam->sample[FATHER]);
+            if ( !khash_str2int_has_key(name2idx, col_ends[2]+1) )
+                lines = add_sample(name2idx, lines, &nlines, &mlines, col_ends[2]+1, 'F', &fam->sample[MOTHER]);
+
+            khash_str2int_get(name2idx, col_ends[0]+1, &fam->sample[CHILD]);
+            khash_str2int_get(name2idx, col_ends[1]+1, &fam->sample[FATHER]);
+            khash_str2int_get(name2idx, col_ends[2]+1, &fam->sample[MOTHER]);
         }
     }
     free(str.s);
+    free(fam_str.s);
+    khash_str2int_destroy_free(name2idx);
 
-    if ( i!=_n ) // not a ped file
+    if ( i!=nvals ) // not a ped file
     {
-        if ( i>0 ) error("Could not parse the samples, thought it was PED format, some rows have 5 columns?!\n");
+        if ( i>0 ) error("Could not parse samples, not a PED format.\n");
         return NULL;
     }
-    assert( n==_n );
-    for (i=0; i<call->nfams; i++)
-        assert( call->fams[i].sample[0]>=0 && call->fams[i].sample[1]>=0 && call->fams[i].sample[2]>=0 ); // multiple childs, not a trio
-
-    return sam;
+    *nsmpl = nlines;
+    return lines;
 }
 
 
@@ -200,44 +256,80 @@ static char **parse_ped_samples(call_t *call, char **vals, int _n)
  *  Alternatively, if no such file exists, the file name is interpreted
  *  as a comma-separated list of samples. When ploidy is not present,
  *  the default ploidy 2 is assumed.
- *
- *  Returns an array of sample names, where the byte value just after \0
- *  indicates the ploidy.
  */
-static char **read_samples(call_t *call, const char *fn, int is_file, int *_n)
+static void set_samples(args_t *args, const char *fn, int is_file)
 {
-    int i, n;
-    char **vals = hts_readlist(fn, is_file, &n);
-    if ( !vals ) error("Could not read the file: %s\n", fn);
+    int i, nlines;
+    char **lines = hts_readlist(fn, is_file, &nlines);
+    if ( !lines ) error("Could not read the file: %s\n", fn);
 
-    char **smpls = parse_ped_samples(call, vals, n);
-    if ( !smpls )
+    int nsmpls;
+    char **smpls = parse_ped_samples(&args->aux, lines, nlines, &nsmpls);
+    if ( smpls )
     {
-        smpls = (char**) malloc(sizeof(char*)*n);
-        for (i=0; i<n; i++)
-        {
-            char *s = vals[i];
-            while ( *s && !isspace(*s) ) s++;
-            int len = s-vals[i];
-            smpls[i] = (char*) malloc(len+2);
-            strncpy(smpls[i],vals[i],len);
-            smpls[i][len] = 0;
-            while ( *s && isspace(*s) ) s++;
-            int x = 2;
-            if ( *s )
-            {
-                x = (int)s[0] - '0'; // Convert ASCII digit to decimal
-                if (x != 0 && x != 1 && x != 2) error("Ploidy can only be 0, 1 or 2: %s\n", vals[i]);
-            }
-            smpls[i][len+1] = x;
-        }
+        for (i=0; i<nlines; i++) free(lines[i]);
+        free(lines);
+        lines = smpls;
+        nlines = nsmpls;
     }
 
-    for (i=0; i<n; i++) free(vals[i]);
-    free(vals);
+    args->samples_map = (int*) malloc(sizeof(int)*bcf_hdr_nsamples(args->aux.hdr)); // for subsetting
+    args->sample2sex  = (int*) malloc(sizeof(int)*bcf_hdr_nsamples(args->aux.hdr));
+    int dflt_sex_id = ploidy_add_sex(args->ploidy, "F");
+    for (i=0; i<bcf_hdr_nsamples(args->aux.hdr); i++) args->sample2sex[i] = dflt_sex_id;
 
-    *_n = n;
-    return smpls;
+    int *old2new = (int*) malloc(sizeof(int)*bcf_hdr_nsamples(args->aux.hdr));
+    for (i=0; i<bcf_hdr_nsamples(args->aux.hdr); i++) old2new[i] = -1;
+
+    int nsmpl = 0, map_needed = 0;
+    for (i=0; i<nlines; i++)
+    {
+        char *ss = lines[i];
+        while ( *ss && isspace(*ss) ) ss++;
+        if ( !*ss ) error("Could not parse: %s\n", lines[i]);
+        if ( *ss=='#' ) continue;
+        char *se = ss;
+        while ( *se && !isspace(*se) ) se++;
+        char x = *se, *xptr = se; *se = 0;
+
+        int ismpl = bcf_hdr_id2int(args->aux.hdr, BCF_DT_SAMPLE, ss);
+        if ( ismpl < 0 ) { fprintf(stderr,"Warning: No such sample in the VCF: %s\n",ss); continue; }
+
+        ss = se+1;
+        while ( *ss && isspace(*ss) ) ss++;
+        if ( !*ss ) ss = "2";   // default ploidy
+        se = ss;
+        while ( *se && !isspace(*se) ) se++;
+        if ( se==ss ) { *xptr = x; error("Could not parse: \"%s\"\n", lines[i]); }
+
+        if ( ss[1]==0 && (ss[0]=='0' || ss[0]=='1' || ss[0]=='2') )
+            args->sample2sex[nsmpl] = -1*(ss[0]-'0');
+        else
+            args->sample2sex[nsmpl] = ploidy_add_sex(args->ploidy, ss);
+
+        if ( ismpl!=nsmpl ) map_needed = 1;
+        args->samples_map[nsmpl] = ismpl;
+        old2new[ismpl] = nsmpl;
+        nsmpl++;
+    }
+
+    for (i=0; i<args->aux.nfams; i++)
+    {
+        int j, nmiss = 0;
+        family_t *fam = &args->aux.fams[i];
+        for (j=0; j<3; j++)
+        {
+            fam->sample[i] = old2new[fam->sample[i]];
+            if ( fam->sample[i]<0 ) nmiss++;
+        }
+        assert( nmiss==0 || nmiss==3 );
+    }
+    free(old2new);
+
+    if ( !map_needed ) { free(args->samples_map); args->samples_map = NULL; }
+
+    args->nsamples = nsmpl;
+    args->samples = lines;
 }
 
 static void init_missed_line(args_t *args)
@@ -259,8 +351,6 @@ static void print_missed_line(bcf_sr_regions_t *regs, void *data)
     call_t *call = &args->aux;
     bcf1_t *missed = args->missed_line;
 
-    if ( args->flag & CF_GVCF ) error("todo: Combine --gvcf and --insert-missed\n");
-
     char *ss = regs->line.s;
     int i = 0;
     while ( i<args->aux.srs->targets_als-1 && *ss )
@@ -296,16 +386,47 @@ static void init_data(args_t *args)
     if ( args->regions )
     {
         if ( bcf_sr_set_regions(args->aux.srs, args->regions, args->regions_is_file)<0 )
-            error("Failed to read the targets: %s\n", args->regions);
+            error("Failed to read the regions: %s\n", args->regions);
     }
 
-    int i;
     if ( !bcf_sr_add_reader(args->aux.srs, args->bcf_fname) ) error("Failed to open %s: %s\n", args->bcf_fname,bcf_sr_strerror(args->aux.srs->errnum));
+    args->aux.hdr = bcf_sr_get_header(args->aux.srs,0);
 
-    if ( args->nsamples && args->nsamples != bcf_hdr_nsamples(args->aux.srs->readers[0].header) )
+    int i;
+    if ( args->samples_fname )
     {
-        args->samples_map = (int *) malloc(sizeof(int)*args->nsamples);
-        args->aux.hdr = bcf_hdr_subset(args->aux.srs->readers[0].header, args->nsamples, args->samples, args->samples_map);
+        set_samples(args, args->samples_fname, args->samples_is_file);
+        if ( args->aux.flag&CALL_CONSTR_TRIO )
+        {
+            if ( 3*args->aux.nfams!=args->nsamples ) error("Expected only trios in %s, sorry!\n", args->samples_fname);
+            fprintf(stderr,"Detected %d samples in %d trio families\n", args->nsamples,args->aux.nfams);
+        }
+    }
+    if ( args->ploidy  )
+    {
+        args->nsex = ploidy_nsex(args->ploidy);
+        args->sex2ploidy = (int*) calloc(args->nsex,sizeof(int));
+        args->sex2ploidy_prev = (int*) calloc(args->nsex,sizeof(int));
+        if ( !args->nsamples )
+        {
+            args->nsamples = bcf_hdr_nsamples(args->aux.hdr);
+            args->sample2sex = (int*) malloc(sizeof(int)*args->nsamples);
+            for (i=0; i<args->nsamples; i++) args->sample2sex[i] = 0;
+        }
+    }
+    if ( args->nsamples )
+    {
+        args->aux.ploidy = (uint8_t*) malloc(args->nsamples);
+        for (i=0; i<args->nsamples; i++) args->aux.ploidy[i] = 2;
+        for (i=0; i<args->nsex; i++) args->sex2ploidy_prev[i] = 2;
+    }
+
+    if ( args->gvcf ) 
+        gvcf_update_header(args->gvcf, args->aux.hdr);
+
+    if ( args->samples_map )
+    {
+        args->aux.hdr = bcf_hdr_subset(bcf_sr_get_header(args->aux.srs,0), args->nsamples, args->samples, args->samples_map);
         if ( !args->aux.hdr ) error("Error occurred while subsetting samples\n");
         for (i=0; i<args->nsamples; i++)
             if ( args->samples_map[i]<0 ) error("No such sample: %s\n", args->samples[i]);
@@ -313,45 +434,18 @@ static void init_data(args_t *args)
     }
     else
     {
-        args->aux.hdr = bcf_hdr_dup(args->aux.srs->readers[0].header);
-        for (i=0; i<args->nsamples; i++)
-            if ( bcf_hdr_id2int(args->aux.hdr,BCF_DT_SAMPLE,args->samples[i])<0 )
-                error("No such sample: %s\n", args->samples[i]);
-    }
-
-    // Reorder ploidy and family indexes to match mpileup's output and exclude samples which are not available
-    if ( args->aux.ploidy )
-    {
-        for (i=0; i<args->aux.nfams; i++)
+        args->aux.hdr = bcf_hdr_dup(bcf_sr_get_header(args->aux.srs,0));
+        if ( args->samples )
         {
-            int j;
-            for (j=0; j<3; j++)
-            {
-                int k = bcf_hdr_id2int(args->aux.hdr, BCF_DT_SAMPLE, args->samples[ args->aux.fams[i].sample[j] ]);
-                if ( k<0 ) error("No such sample: %s\n", args->samples[ args->aux.fams[i].sample[j] ]);
-                args->aux.fams[i].sample[j] = k;
-            }
+            for (i=0; i<args->nsamples; i++)
+                if ( bcf_hdr_id2int(args->aux.hdr,BCF_DT_SAMPLE,args->samples[i])<0 )
+                    error("No such sample: %s\n", args->samples[i]);
         }
-        uint8_t *ploidy = (uint8_t*) calloc(bcf_hdr_nsamples(args->aux.hdr), 1);
-        for (i=0; i<args->nsamples; i++)    // i index in -s sample list
-        {
-            int j = bcf_hdr_id2int(args->aux.hdr, BCF_DT_SAMPLE, args->samples[i]);     // j index in the output VCF / subset VCF
-            if ( j<0 )
-            {
-                fprintf(stderr,"Warning: no such sample: \"%s\"\n", args->samples[i]);
-                continue;
-            }
-            ploidy[j] = args->aux.ploidy[i];
-        }
-        args->nsamples = bcf_hdr_nsamples(args->aux.hdr);
-        for (i=0; i<args->nsamples; i++)
-            assert( ploidy[i]==0 || ploidy[i]==1 || ploidy[i]==2 );
-        free(args->aux.ploidy);
-        args->aux.ploidy = ploidy;
     }
 
     args->out_fh = hts_open(args->output_fname, hts_bcf_wmode(args->output_type));
     if ( args->out_fh == NULL ) error("Can't write to \"%s\": %s\n", args->output_fname, strerror(errno));
+    if ( args->n_threads ) hts_set_threads(args->out_fh, args->n_threads);
 
     if ( args->flag & CF_QCALL )
         return;
@@ -362,23 +456,10 @@ static void init_data(args_t *args)
     if ( args->flag & CF_CCALL )
         ccall_init(&args->aux);
 
-    if ( args->flag&CF_GVCF )
-    {
-        bcf_hdr_append(args->aux.hdr,"##INFO=<ID=END,Number=1,Type=Integer,Description=\"End position of the variant described in this record\">");
-        args->gvcf.rid  = -1;
-        args->gvcf.line = bcf_init1();
-        args->gvcf.gt   = (int32_t*) malloc(2*sizeof(int32_t)*bcf_hdr_nsamples(args->aux.hdr));
-        for (i=0; i<bcf_hdr_nsamples(args->aux.hdr); i++)
-        {
-            args->gvcf.gt[2*i+0] = bcf_gt_unphased(0);
-            args->gvcf.gt[2*i+1] = bcf_gt_unphased(0);
-        }
-    }
-
     bcf_hdr_remove(args->aux.hdr, BCF_HL_INFO, "QS");
     bcf_hdr_remove(args->aux.hdr, BCF_HL_INFO, "I16");
 
-    bcf_hdr_append_version(args->aux.hdr, args->argc, args->argv, "bcftools_call");
+    if (args->record_cmd_line) bcf_hdr_append_version(args->aux.hdr, args->argc, args->argv, "bcftools_call");
     bcf_hdr_write(args->out_fh, args->aux.hdr);
 
     if ( args->flag&CF_INS_MISSED ) init_missed_line(args);
@@ -390,19 +471,24 @@ static void destroy_data(args_t *args)
     else if ( args->flag & CF_MCALL ) mcall_destroy(&args->aux);
     else if ( args->flag & CF_QCALL ) qcall_destroy(&args->aux);
     int i;
-    for (i=0; i<args->nsamples; i++) free(args->samples[i]);
+    if ( args->samples )
+    {
+        for (i=0; i<args->nsamples; i++) free(args->samples[i]);
+    }
     if ( args->aux.fams )
     {
         for (i=0; i<args->aux.nfams; i++) free(args->aux.fams[i].name);
         free(args->aux.fams);
     }
     if ( args->missed_line ) bcf_destroy(args->missed_line);
-    if ( args->gvcf.line ) bcf_destroy(args->gvcf.line);
-    free(args->gvcf.gt);
-    free(args->gvcf.dp);
+    ploidy_destroy(args->ploidy);
+    free(args->sex2ploidy);
+    free(args->sex2ploidy_prev);
     free(args->samples);
     free(args->samples_map);
+    free(args->sample2sex);
     free(args->aux.ploidy);
+    if ( args->gvcf ) gvcf_destroy(args->gvcf);
     bcf_hdr_destroy(args->aux.hdr);
     hts_close(args->out_fh);
     bcf_sr_destroy(args->aux.srs);
@@ -451,6 +537,59 @@ static int parse_format_flag(const char *str)
     return flag;
 }
 
+static void set_ploidy(args_t *args, bcf1_t *rec)
+{
+    ploidy_query(args->ploidy,(char*)bcf_seqname(args->aux.hdr,rec),rec->pos,args->sex2ploidy,NULL,NULL);
+
+    int i;
+    for (i=0; i<args->nsex; i++)
+        if ( args->sex2ploidy[i]!=args->sex2ploidy_prev[i] ) break;
+
+    if ( i==args->nsex ) return;    // ploidy same as previously
+
+    for (i=0; i<args->nsamples; i++)
+    {
+        if ( args->sample2sex[i]<0 )
+            args->aux.ploidy[i] = -1*args->sample2sex[i];
+        else
+            args->aux.ploidy[i] = args->sex2ploidy[args->sample2sex[i]];
+    }
+
+    int *tmp = args->sex2ploidy; args->sex2ploidy = args->sex2ploidy_prev; args->sex2ploidy_prev = tmp;
+}
+
+ploidy_t *init_ploidy(char *alias)
+{
+    const ploidy_predef_t *pld = ploidy_predefs;
+
+    int detailed = 0, len = strlen(alias);
+    if ( alias[len-1]=='?' ) { detailed = 1; alias[len-1] = 0; }
+
+    while ( pld->alias && strcasecmp(alias,pld->alias) ) pld++;
+
+    if ( !pld->alias )
+    {
+        fprintf(stderr,"Predefined ploidies:\n");
+        pld = ploidy_predefs;
+        while ( pld->alias )
+        {
+            fprintf(stderr,"%s\n   .. %s\n\n", pld->alias,pld->about);
+            if ( detailed )
+                fprintf(stderr,"%s\n", pld->ploidy);
+            pld++;
+        }
+        fprintf(stderr,"Run as --ploidy <alias> (e.g. --ploidy GRCh37).\n");
+        fprintf(stderr,"To see the detailed ploidy definition, append a question mark (e.g. --ploidy GRCh37?).\n");
+        fprintf(stderr,"\n");
+        exit(-1);
+    }
+    else if ( detailed )
+    {
+        fprintf(stderr,"%s", pld->ploidy);
+        exit(-1);
+    }
+    return ploidy_init_string(pld->ploidy,2);
+}
 
 static void usage(args_t *args)
 {
@@ -463,19 +602,23 @@ static void usage(args_t *args)
     fprintf(stderr, "Usage:   bcftools call [options] <in.vcf.gz>\n");
     fprintf(stderr, "\n");
     fprintf(stderr, "File format options:\n");
+    fprintf(stderr, "       --no-version                do not append version and command line to the header\n");
     fprintf(stderr, "   -o, --output <file>             write output to a file [standard output]\n");
     fprintf(stderr, "   -O, --output-type <b|u|z|v>     output type: 'b' compressed BCF; 'u' uncompressed BCF; 'z' compressed VCF; 'v' uncompressed VCF [v]\n");
+    fprintf(stderr, "       --ploidy <assembly>[?]      predefined ploidy, 'list' to print available settings, append '?' for details\n");
+    fprintf(stderr, "       --ploidy-file <file>        space/tab-delimited list of CHROM,FROM,TO,SEX,PLOIDY\n");
     fprintf(stderr, "   -r, --regions <region>          restrict to comma-separated list of regions\n");
     fprintf(stderr, "   -R, --regions-file <file>       restrict to regions listed in a file\n");
     fprintf(stderr, "   -s, --samples <list>            list of samples to include [all samples]\n");
-    fprintf(stderr, "   -S, --samples-file <file>       PED file or a file with optional second column for ploidy (0, 1 or 2) [all samples]\n");
+    fprintf(stderr, "   -S, --samples-file <file>       PED file or a file with an optional column with sex (see man page for details) [all samples]\n");
     fprintf(stderr, "   -t, --targets <region>          similar to -r but streams rather than index-jumps\n");
     fprintf(stderr, "   -T, --targets-file <file>       similar to -R but streams rather than index-jumps\n");
+    fprintf(stderr, "       --threads <int>             number of extra output compression threads [0]\n");
     fprintf(stderr, "\n");
     fprintf(stderr, "Input/output options:\n");
     fprintf(stderr, "   -A, --keep-alts                 keep all possible alternate alleles at variant sites\n");
     fprintf(stderr, "   -f, --format-fields <list>      output format fields: GQ,GP (lowercase allowed) []\n");
-    fprintf(stderr, "   -g, --gvcf <minDP>              output gVCF blocks of homozygous REF calls\n");
+    fprintf(stderr, "   -g, --gvcf <int>,[...]          group non-variant sites into gVCF blocks by minimum per-sample DP\n");
     fprintf(stderr, "   -i, --insert-missed             output also sites missed by mpileup but present in -T\n");
     fprintf(stderr, "   -M, --keep-masked-ref           keep sites with masked reference allele (REF=N)\n");
     fprintf(stderr, "   -V, --skip-variants <type>      skip indels/snps\n");
@@ -488,8 +631,6 @@ static void usage(args_t *args)
     fprintf(stderr, "   -n, --novel-rate <float>,[...]  likelihood of novel mutation for constrained trio calling, see man page for details [1e-8,1e-9,1e-9]\n");
     fprintf(stderr, "   -p, --pval-threshold <float>    variant if P(ref|D)<FLOAT with -c [0.5]\n");
     fprintf(stderr, "   -P, --prior <float>             mutation rate (use bigger for greater sensitivity) [1.1e-3]\n");
-    fprintf(stderr, "   -X, --chromosome-X              haploid output for male samples (requires PED file with -s)\n");
-    fprintf(stderr, "   -Y, --chromosome-Y              haploid output for males and skips females (requires PED file with -s)\n");
 
     // todo (and more)
     // fprintf(stderr, "\nContrast calling and association test options:\n");
@@ -503,7 +644,7 @@ static void usage(args_t *args)
 
 int main_vcfcall(int argc, char *argv[])
 {
-    char *samples_fname = NULL;
+    char *ploidy_fname = NULL, *ploidy = NULL;
     args_t args;
     memset(&args, 0, sizeof(args_t));
     args.argc = argc; args.argv = argv;
@@ -516,51 +657,55 @@ int main_vcfcall(int argc, char *argv[])
     args.flag           = CF_ACGT_ONLY;
     args.output_fname   = "-";
     args.output_type    = FT_VCF;
+    args.n_threads = 0;
+    args.record_cmd_line = 1;
     args.aux.trio_Pm_SNPs = 1 - 1e-8;
     args.aux.trio_Pm_ins  = args.aux.trio_Pm_del  = 1 - 1e-9;
 
-    int i, c, samples_is_file = 0;
-
+    int c;
     static struct option loptions[] =
     {
-        {"help",0,0,'h'},
-        {"gvcf",1,0,'g'},
-        {"format-fields",1,0,'f'},
-        {"output",1,0,'o'},
-        {"output-type",1,0,'O'},
-        {"regions",1,0,'r'},
-        {"regions-file",1,0,'R'},
-        {"samples",1,0,'s'},
-        {"samples-file",1,0,'S'},
-        {"targets",1,0,'t'},
-        {"targets-file",1,0,'T'},
-        {"keep-alts",0,0,'A'},
-        {"insert-missed",0,0,'i'},
-        {"skip-Ns",0,0,'N'},            // now the new default
-        {"keep-masked-refs",0,0,'M'},
-        {"skip-variants",1,0,'V'},
-        {"variants-only",0,0,'v'},
-        {"consensus-caller",0,0,'c'},
-        {"constrain",1,0,'C'},
-        {"multiallelic-caller",0,0,'m'},
-        {"pval-threshold",1,0,'p'},
-        {"prior",1,0,'P'},
-        {"chromosome-X",0,0,'X'},
-        {"chromosome-Y",0,0,'Y'},
-        {"novel-rate",1,0,'n'},
-        {0,0,0,0}
+        {"help",no_argument,NULL,'h'},
+        {"format-fields",required_argument,NULL,'f'},
+        {"gvcf",required_argument,NULL,'g'},
+        {"output",required_argument,NULL,'o'},
+        {"output-type",required_argument,NULL,'O'},
+        {"regions",required_argument,NULL,'r'},
+        {"regions-file",required_argument,NULL,'R'},
+        {"samples",required_argument,NULL,'s'},
+        {"samples-file",required_argument,NULL,'S'},
+        {"targets",required_argument,NULL,'t'},
+        {"targets-file",required_argument,NULL,'T'},
+        {"threads",required_argument,NULL,9},
+        {"keep-alts",no_argument,NULL,'A'},
+        {"insert-missed",no_argument,NULL,'i'},
+        {"skip-Ns",no_argument,NULL,'N'},            // now the new default
+        {"keep-masked-refs",no_argument,NULL,'M'},
+        {"skip-variants",required_argument,NULL,'V'},
+        {"variants-only",no_argument,NULL,'v'},
+        {"consensus-caller",no_argument,NULL,'c'},
+        {"constrain",required_argument,NULL,'C'},
+        {"multiallelic-caller",no_argument,NULL,'m'},
+        {"pval-threshold",required_argument,NULL,'p'},
+        {"prior",required_argument,NULL,'P'},
+        {"novel-rate",required_argument,NULL,'n'},
+        {"ploidy",required_argument,NULL,1},
+        {"ploidy-file",required_argument,NULL,2},
+        {"chromosome-X",no_argument,NULL,'X'},
+        {"chromosome-Y",no_argument,NULL,'Y'},
+        {"no-version",no_argument,NULL,8},
+        {NULL,0,NULL,0}
     };
 
     char *tmp = NULL;
-    while ((c = getopt_long(argc, argv, "h?o:O:r:R:s:S:t:T:ANMV:vcmp:C:XYn:P:f:ig:", loptions, NULL)) >= 0)
+    while ((c = getopt_long(argc, argv, "h?o:O:r:R:s:S:t:T:ANMV:vcmp:C:n:P:f:ig:XY", loptions, NULL)) >= 0)
     {
         switch (c)
         {
-            case 'g':
-                args.flag |= CF_GVCF;
-                args.gvcf.min_dp = strtol(optarg,&tmp,10);
-                if ( *tmp ) error("Could not parse, expected integer argument: -g %s\n", optarg);
-                break;
+            case  2 : ploidy_fname = optarg; break;
+            case  1 : ploidy = optarg; break;
+            case 'X': ploidy = "X"; fprintf(stderr,"Warning: -X will be deprecated, please use --ploidy instead.\n"); break;
+            case 'Y': ploidy = "Y"; fprintf(stderr,"Warning: -Y will be deprecated, please use --ploidy instead.\n"); break;
             case 'f': args.aux.output_tags |= parse_format_flag(optarg); break;
             case 'M': args.flag &= ~CF_ACGT_ONLY; break;     // keep sites where REF is N
             case 'N': args.flag |= CF_ACGT_ONLY; break;      // omit sites where first base in REF is N (the new default)
@@ -568,6 +713,10 @@ int main_vcfcall(int argc, char *argv[])
             case 'c': args.flag |= CF_CCALL; break;          // the original EM based calling method
             case 'i': args.flag |= CF_INS_MISSED; break;
             case 'v': args.aux.flag |= CALL_VARONLY; break;
+            case 'g': 
+                args.gvcf = gvcf_init(optarg);
+                if ( !args.gvcf ) error("Could not parse: --gvcf %s\n", optarg);
+                break;
             case 'o': args.output_fname = optarg; break;
             case 'O':
                       switch (optarg[0]) {
@@ -583,8 +732,6 @@ int main_vcfcall(int argc, char *argv[])
                       else if ( !strcasecmp(optarg,"trio") ) args.aux.flag |= CALL_CONSTR_TRIO;
                       else error("Unknown argument to -C: \"%s\"\n", optarg);
                       break;
-            case 'X': args.aux.flag |= CALL_CHR_X; break;
-            case 'Y': args.aux.flag |= CALL_CHR_Y; break;
             case 'V':
                       if ( !strcasecmp(optarg,"snps") ) args.flag |= CF_INDEL_ONLY;
                       else if ( !strcasecmp(optarg,"indels") ) args.flag |= CF_NO_INDEL;
@@ -603,11 +750,17 @@ int main_vcfcall(int argc, char *argv[])
             case 'R': args.regions = optarg; args.regions_is_file = 1; break;
             case 't': args.targets = optarg; break;
             case 'T': args.targets = optarg; args.targets_is_file = 1; break;
-            case 's': samples_fname = optarg; break;
-            case 'S': samples_fname = optarg; samples_is_file = 1; break;
+            case 's': args.samples_fname = optarg; break;
+            case 'S': args.samples_fname = optarg; args.samples_is_file = 1; break;
+            case  9 : args.n_threads = strtol(optarg, 0, 0); break;
+            case  8 : args.record_cmd_line = 0; break;
             default: usage(&args);
         }
     }
+    // Sanity check options and initialize
+    if ( ploidy_fname ) args.ploidy = ploidy_init(ploidy_fname, 2);
+    else if ( ploidy ) args.ploidy = init_ploidy(ploidy);
+
     if ( optind>=argc )
     {
         if ( !isatty(fileno((FILE *)stdin)) ) args.bcf_fname = "-";  // reading from stdin
@@ -615,34 +768,24 @@ int main_vcfcall(int argc, char *argv[])
     }
     else args.bcf_fname = argv[optind++];
 
-    // Sanity check options and initialize
-    if ( samples_fname )
+    if ( !ploidy_fname && !ploidy )
     {
-        args.samples = read_samples(&args.aux, samples_fname, samples_is_file, &args.nsamples);
-        args.aux.ploidy = (uint8_t*) calloc(args.nsamples+1, 1);
-        args.aux.all_diploid = 1;
-        for (i=0; i<args.nsamples; i++)
-        {
-            args.aux.ploidy[i] = args.samples[i][strlen(args.samples[i]) + 1];
-            if ( args.aux.ploidy[i]!=2 ) args.aux.all_diploid = 0;
-        }
-    }
-    if ( args.flag & CF_GVCF )
-    {
-        // Force some flags to avoid unnecessary branching
-        args.aux.flag &= ~CALL_KEEPALT;
-        args.aux.flag |= CALL_VARONLY;
+        fprintf(stderr,"Note: Neither --ploidy nor --ploidy-file given, assuming all sites are diploid\n");
+        args.ploidy = ploidy_init_string("",2);
     }
+
+    if ( !args.ploidy ) error("Could not initialize ploidy\n");
     if ( (args.flag & CF_CCALL ? 1 : 0) + (args.flag & CF_MCALL ? 1 : 0) + (args.flag & CF_QCALL ? 1 : 0) > 1 ) error("Only one of -c or -m options can be given\n");
     if ( !(args.flag & CF_CCALL) && !(args.flag & CF_MCALL) && !(args.flag & CF_QCALL) ) error("Expected -c or -m option\n");
+    if ( (args.flag & CF_CCALL ? 1: 0) && args.gvcf ) error("gvcf -g option not functional with -c calling mode yet\n");
     if ( args.aux.n_perm && args.aux.ngrp1_samples<=0 ) error("Expected -1 with -U\n");    // not sure about this, please fix
     if ( args.aux.flag & CALL_CONSTR_ALLELES )
     {
         if ( !args.targets ) error("Expected -t or -T with \"-C alleles\"\n");
         if ( !(args.flag & CF_MCALL) ) error("The \"-C alleles\" mode requires -m\n");
     }
-    if ( args.aux.flag & CALL_CHR_X && args.aux.flag & CALL_CHR_Y ) error("Only one of -X or -Y should be given\n");
     if ( args.flag & CF_INS_MISSED && !(args.aux.flag&CALL_CONSTR_ALLELES) ) error("The -i option requires -C alleles\n");
+    if ( args.aux.flag&CALL_VARONLY && args.gvcf ) error("The two options cannot be combined: --variants-only and --gvcf\n");
     init_data(&args);
 
     while ( bcf_sr_next_line(args.aux.srs) )
@@ -652,29 +795,29 @@ int main_vcfcall(int argc, char *argv[])
         bcf_unpack(bcf_rec, BCF_UN_STR);
 
         // Skip unwanted sites
-        if ( args.aux.flag & CALL_VARONLY )
-        {
-            int is_ref = 0;
-            if ( bcf_rec->n_allele==1 ) is_ref = 1;     // not a variant
-            else if ( bcf_rec->n_allele==2 )
-            {
-                // second allele is mpileup's X, not a variant
-                if ( bcf_rec->d.allele[1][0]=='X' ) is_ref = 1;
-                else if ( bcf_rec->d.allele[1][0]=='<' && bcf_rec->d.allele[1][1]=='X' && bcf_rec->d.allele[1][2]=='>' ) is_ref = 1;
-                else if ( bcf_rec->d.allele[1][0]=='<' && bcf_rec->d.allele[1][1]=='*' && bcf_rec->d.allele[1][2]=='>' ) is_ref = 1;
-            }
-            if ( is_ref )
-            {
-                // gVCF output
-                if ( args.flag & CF_GVCF ) gvcf_write(args.out_fh, &args.gvcf, args.aux.hdr, bcf_rec, 1);
-                continue;
-            }
-        }
-        if ( (args.flag & CF_INDEL_ONLY) && bcf_is_snp(bcf_rec) ) continue;    // not an indel
-        if ( (args.flag & CF_NO_INDEL) && !bcf_is_snp(bcf_rec) ) continue;     // not a SNP
+        int i, is_indel = bcf_is_snp(bcf_rec) ? 0 : 1;
+        if ( (args.flag & CF_INDEL_ONLY) && !is_indel ) continue;
+        if ( (args.flag & CF_NO_INDEL) && is_indel ) continue;
         if ( (args.flag & CF_ACGT_ONLY) && (bcf_rec->d.allele[0][0]=='N' || bcf_rec->d.allele[0][0]=='n') ) continue;   // REF[0] is 'N'
 
+        // Which allele is symbolic? All SNPs should have it, but not indels
+        args.aux.unseen = 0;
+        for (i=1; i<bcf_rec->n_allele; i++)
+        {
+            if ( bcf_rec->d.allele[i][0]=='X' ) { args.aux.unseen = i; break; }  // old X
+            if ( bcf_rec->d.allele[i][0]=='<' )
+            {
+                if ( bcf_rec->d.allele[i][1]=='X' && bcf_rec->d.allele[i][2]=='>' ) { args.aux.unseen = i; break; } // old <X>
+                if ( bcf_rec->d.allele[i][1]=='*' && bcf_rec->d.allele[i][2]=='>' ) { args.aux.unseen = i; break; } // new <*>
+            }
+        }
+        int is_ref = (bcf_rec->n_allele==1 || (bcf_rec->n_allele==2 && args.aux.unseen>0)) ? 1 : 0;
+
+        if ( is_ref && args.aux.flag&CALL_VARONLY )
+            continue;
+
         bcf_unpack(bcf_rec, BCF_UN_ALL);
+        if ( args.nsex ) set_ploidy(&args, bcf_rec);
 
         // Various output modes: QCall output (todo)
         if ( args.flag & CF_QCALL )
@@ -691,18 +834,14 @@ int main_vcfcall(int argc, char *argv[])
             ret = ccall(&args.aux, bcf_rec);
         if ( ret==-1 ) error("Something is wrong\n");
 
-        // gVCF output
-        if ( args.flag & CF_GVCF )
-        {
-            gvcf_write(args.out_fh, &args.gvcf, args.aux.hdr, bcf_rec, ret?0:1);
-            continue;
-        }
-
         // Normal output
-        if ( (args.aux.flag & CALL_VARONLY) && ret==0 ) continue;     // not a variant
-        bcf_write1(args.out_fh, args.aux.hdr, bcf_rec);
+        if ( (args.aux.flag & CALL_VARONLY) && ret==0 && !args.gvcf ) continue;     // not a variant
+        if ( args.gvcf )
+            bcf_rec = gvcf_write(args.gvcf, args.out_fh, args.aux.hdr, bcf_rec, ret==1?1:0);
+        if ( bcf_rec )
+            bcf_write1(args.out_fh, args.aux.hdr, bcf_rec);
     }
-    if ( args.flag & CF_GVCF ) gvcf_write(args.out_fh, &args.gvcf, args.aux.hdr, NULL, 0);
+    if ( args.gvcf ) gvcf_write(args.gvcf, args.out_fh, args.aux.hdr, NULL, 0);
     if ( args.flag & CF_INS_MISSED ) bcf_sr_regions_flush(args.aux.srs->targets);
     destroy_data(&args);
     return 0;
