@@ -1,6 +1,6 @@
 /*  vcfstats.c -- Produces stats which can be plotted using plot-vcfstats.
 
-    Copyright (C) 2012-2014 Genome Research Ltd.
+    Copyright (C) 2012-2015 Genome Research Ltd.
 
     Author: Petr Danecek <pd3@sanger.ac.uk>
 
@@ -38,11 +38,17 @@ THE SOFTWARE.  */
 #include <htslib/faidx.h>
 #include <inttypes.h>
 #include "bcftools.h"
+#include "filter.h"
+
+// Logic of the filters: include or exclude sites which match the filters?
+#define FLT_INCLUDE 1
+#define FLT_EXCLUDE 2
 
 #define HWE_STATS 1
 #define QUAL_STATS 1
 #define IRC_STATS 1
 #define IRC_RLEN 10
+#define NA_STRING "0"
 
 typedef struct
 {
@@ -63,7 +69,18 @@ idist_t;
 
 typedef struct
 {
-    int n_snps, n_indels, n_mnps, n_others, n_mals, n_snp_mals, n_records;
+    double x;
+    double x2;
+    double y;
+    double y2;
+    double xy;
+    double n;
+}
+smpl_r_t;
+
+typedef struct
+{
+    int n_snps, n_indels, n_mnps, n_others, n_mals, n_snp_mals, n_records, n_noalts;
     int *af_ts, *af_tv, *af_snps;   // first bin of af_* stats are singletons
     #if HWE_STATS
         int *af_hwe;
@@ -80,6 +97,7 @@ typedef struct
     int in_frame, out_frame, na_frame, in_frame_alt1, out_frame_alt1, na_frame_alt1;
     int subst[15];
     int *smpl_hets, *smpl_homRR, *smpl_homAA, *smpl_ts, *smpl_tv, *smpl_indels, *smpl_ndp, *smpl_sngl;
+    int *smpl_indel_hets, *smpl_indel_homs;
     int *smpl_frm_shifts; // not-applicable, in-frame, out-frame
     unsigned long int *smpl_dp;
     idist_t dp, dp_sites;
@@ -133,6 +151,14 @@ typedef struct
     char **argv, *exons_fname, *regions_list, *samples_list, *targets_list;
     int argc, verbose_sites, first_allele_only, samples_is_file;
     int split_by_id, nstats;
+
+    filter_t *filter[2];
+    char *filter_str;
+    int filter_logic;   // include or exclude sites which match the filters? One of FLT_INCLUDE/FLT_EXCLUDE
+
+    // Per Sample r working data arrays of size equal to number of samples
+    smpl_r_t* smpl_r_snps;
+    smpl_r_t* smpl_r_indels;
 }
 args_t;
 
@@ -372,6 +398,13 @@ static void init_stats(args_t *args)
     args->nstats = args->files->nreaders==1 ? 1 : 3;
     if ( args->split_by_id ) args->nstats = 2;
 
+    if ( args->filter_str )
+    {
+        args->filter[0] = filter_init(bcf_sr_get_header(args->files,0), args->filter_str);
+        if ( args->files->nreaders==2 )
+            args->filter[1] = filter_init(bcf_sr_get_header(args->files,1), args->filter_str);
+    }
+
     // AF corresponds to AC but is more robust for mixture of haploid and diploid GTs
     args->m_af = 101;
     for (i=0; i<args->files->nreaders; i++)
@@ -397,6 +430,8 @@ static void init_stats(args_t *args)
         args->af_gts_indels   = (gtcmp_t *) calloc(args->m_af,sizeof(gtcmp_t));
         args->smpl_gts_snps   = (gtcmp_t *) calloc(args->files->n_smpl,sizeof(gtcmp_t));
         args->smpl_gts_indels = (gtcmp_t *) calloc(args->files->n_smpl,sizeof(gtcmp_t));
+        args->smpl_r_snps = (smpl_r_t*) calloc(args->files->n_smpl, sizeof(smpl_r_t));
+        args->smpl_r_indels = (smpl_r_t*) calloc(args->files->n_smpl, sizeof(smpl_r_t));
     }
     for (i=0; i<args->nstats; i++)
     {
@@ -420,6 +455,8 @@ static void init_stats(args_t *args)
             stats->smpl_hets   = (int *) calloc(args->files->n_smpl,sizeof(int));
             stats->smpl_homAA  = (int *) calloc(args->files->n_smpl,sizeof(int));
             stats->smpl_homRR  = (int *) calloc(args->files->n_smpl,sizeof(int));
+            stats->smpl_indel_hets = (int *) calloc(args->files->n_smpl,sizeof(int));
+            stats->smpl_indel_homs = (int *) calloc(args->files->n_smpl,sizeof(int));
             stats->smpl_ts     = (int *) calloc(args->files->n_smpl,sizeof(int));
             stats->smpl_tv     = (int *) calloc(args->files->n_smpl,sizeof(int));
             stats->smpl_indels = (int *) calloc(args->files->n_smpl,sizeof(int));
@@ -497,6 +534,8 @@ static void destroy_stats(args_t *args)
         if (stats->smpl_hets) free(stats->smpl_hets);
         if (stats->smpl_homAA) free(stats->smpl_homAA);
         if (stats->smpl_homRR) free(stats->smpl_homRR);
+        if (stats->smpl_indel_homs) free(stats->smpl_indel_homs);
+        if (stats->smpl_indel_hets) free(stats->smpl_indel_hets);
         if (stats->smpl_ts) free(stats->smpl_ts);
         if (stats->smpl_tv) free(stats->smpl_tv);
         if (stats->smpl_indels) free(stats->smpl_indels);
@@ -517,13 +556,17 @@ static void destroy_stats(args_t *args)
     for (j=0; j<args->nusr; j++) free(args->usr[j].tag);
     free(args->usr);
     free(args->tmp_frm);
-    if (args->tmp_iaf) free(args->tmp_iaf);
+    free(args->tmp_iaf);
     if (args->exons) bcf_sr_regions_destroy(args->exons);
-    if (args->af_gts_snps) free(args->af_gts_snps);
-    if (args->af_gts_indels) free(args->af_gts_indels);
-    if (args->smpl_gts_snps) free(args->smpl_gts_snps);
-    if (args->smpl_gts_indels) free(args->smpl_gts_indels);
+    free(args->af_gts_snps);
+    free(args->af_gts_indels);
+    free(args->smpl_gts_snps);
+    free(args->smpl_gts_indels);
+    free(args->smpl_r_snps);
+    free(args->smpl_r_indels);
     if (args->indel_ctx) indel_ctx_destroy(args->indel_ctx);
+    if (args->filter[0]) filter_destroy(args->filter[0]);
+    if (args->filter[1]) filter_destroy(args->filter[1]);
 }
 
 static void init_iaf(args_t *args, bcf_sr_t *reader)
@@ -790,7 +833,7 @@ static void do_sample_stats(args_t *args, stats_t *stats, bcf_sr_t *reader, int
                     case GT_HOM_AA: nalt_tot++; break;
                 }
             #endif
-            if ( line_type&VCF_SNP )
+            if ( line_type&VCF_SNP || line_type==VCF_REF )  // count ALT=. as SNP
             {
                 if ( gt == GT_HET_RA ) stats->smpl_hets[is]++;
                 else if ( gt == GT_HET_AA ) stats->smpl_hets[is]++;
@@ -808,7 +851,12 @@ static void do_sample_stats(args_t *args, stats_t *stats, bcf_sr_t *reader, int
             }
             if ( line_type&VCF_INDEL )
             {
-                if ( gt != GT_HOM_RR ) stats->smpl_indels[is]++;
+                if ( gt != GT_HOM_RR )
+                {
+                    stats->smpl_indels[is]++;
+                    if ( gt==GT_HET_RA || gt==GT_HET_AA ) stats->smpl_indel_hets[is]++;
+                    else if ( gt==GT_HOM_AA ) stats->smpl_indel_homs[is]++;
+                }
                 if ( stats->smpl_frm_shifts )
                 {
                     assert( ial<line->n_allele && jal<line->n_allele );
@@ -868,8 +916,28 @@ static void do_sample_stats(args_t *args, stats_t *stats, bcf_sr_t *reader, int
         gtcmp_t *af_stats = line_type&VCF_SNP ? args->af_gts_snps : args->af_gts_indels;
         gtcmp_t *smpl_stats = line_type&VCF_SNP ? args->smpl_gts_snps : args->smpl_gts_indels;
 
+        //
+        // Calculates r squared
+        // x is mean dosage of x at given site
+        // x2 is mean squared dosage of x at given site
+        // y is mean dosage of x at given site
+        // y2 is mean squared dosage of x at given site
+        // xy is mean dosage of x*y at given site
+        // r2sum += (xy - x*y)^2 / ( (x2 - x^2) * (y2 - y^2) )
+        // r2n is number of sites considered
+        // output as r2sum/r2n for each AF bin
         int r2n = 0;
         float x = 0, y = 0, xy = 0, x2 = 0, y2 = 0;
+        // Select smpl_r
+        smpl_r_t *smpl_r = NULL;
+        if (line_type&VCF_SNP)
+        {
+            smpl_r = args->smpl_r_snps;
+        }
+        else if (line_type&VCF_INDEL)
+        {
+            smpl_r = args->smpl_r_indels;
+        }
         for (is=0; is<files->n_smpl; is++)
         {
             // Simplified comparison: only 0/0, 0/1, 1/1 is looked at as the identity of
@@ -903,7 +971,18 @@ static void do_sample_stats(args_t *args, stats_t *stats, bcf_sr_t *reader, int
                 smpl_stats[is].mm[idx]++;
             }
 
+            // Now do it across samples
+
+            if (smpl_r) {
+                smpl_r[is].xy += dsg0*dsg1;
+                smpl_r[is].x += dsg0;
+                smpl_r[is].x2 += dsg0*dsg0;
+                smpl_r[is].y += dsg1;
+                smpl_r[is].y2 += dsg1*dsg1;
+                ++(smpl_r[is].n);
+            }
         }
+
         if ( r2n )
         {
             x /= r2n; y /= r2n; x2 /= r2n; y2 /= r2n; xy /= r2n;
@@ -951,15 +1030,26 @@ static void do_vcf_stats(args_t *args)
     {
         bcf_sr_t *reader = NULL;
         bcf1_t *line = NULL;
-        int ret = 0, i;
+        int ret = 0, i, pass = 1;
         for (i=0; i<files->nreaders; i++)
         {
             if ( !bcf_sr_has_line(files,i) ) continue;
+            if ( args->filter[i] )
+            {
+                int is_ok = filter_test(args->filter[i], bcf_sr_get_line(files,i), NULL);
+                if ( args->filter_logic & FLT_EXCLUDE ) is_ok = is_ok ? 0 : 1;
+                if ( !is_ok ) { pass = 0; break; }
+            }
             ret |= 1<<i;
-            if ( reader ) continue;
-            reader = &files->readers[i];
-            line = files->readers[i].buffer[0];
+            if ( !reader )
+            {
+                reader = &files->readers[i];
+                line = bcf_sr_get_line(files,i);
+            }
+
         }
+        if ( !pass ) continue;
+
         int line_type = bcf_get_variant_types(line);
         init_iaf(args, reader);
 
@@ -969,6 +1059,8 @@ static void do_vcf_stats(args_t *args)
 
         stats->n_records++;
 
+        if ( line_type==VCF_REF )
+            stats->n_noalts++;
         if ( line_type&VCF_SNP )
             do_snp_stats(args, stats, reader);
         if ( line_type&VCF_INDEL )
@@ -1045,6 +1137,7 @@ static void print_stats(args_t *args)
     {
         stats_t *stats = &args->stats[id];
         printf("SN\t%d\tnumber of records:\t%d\n", id, stats->n_records);
+        printf("SN\t%d\tnumber of no-ALTs:\t%d\n", id, stats->n_noalts);
         printf("SN\t%d\tnumber of SNPs:\t%d\n", id, stats->n_snps);
         printf("SN\t%d\tnumber of MNPs:\t%d\n", id, stats->n_mnps);
         printf("SN\t%d\tnumber of indels:\t%d\n", id, stats->n_indels);
@@ -1229,23 +1322,37 @@ static void print_stats(args_t *args)
         for (x=0; x<2; x++)
         {
             gtcmp_t *stats;
+            smpl_r_t *smpl_r_array;
             if ( x==0 )
             {
-                printf("# GCcS, Genotype concordance by sample (SNPs)\n# GCsS\t[2]id\t[3]sample\t[4]non-reference discordance rate\t[5]RR Hom matches\t[6]RA Het matches\t[7]AA Hom matches\t[8]RR Hom mismatches\t[9]RA Het mismatches\t[10]AA Hom mismatches\n");
+                printf("# GCsS, Genotype concordance by sample (SNPs)\n# GCsS\t[2]id\t[3]sample\t[4]non-reference discordance rate\t[5]RR Hom matches\t[6]RA Het matches\t[7]AA Hom matches\t[8]RR Hom mismatches\t[9]RA Het mismatches\t[10]AA Hom mismatches\t[11]dosage r-squared\n");
                 stats = args->smpl_gts_snps;
+                smpl_r_array = args->smpl_r_snps;
             }
             else
             {
-                printf("# GCiS, Genotype concordance by sample (indels)\n# GCiS\t[2]id\t[3]sample\t[4]non-reference discordance rate\t[5]RR Hom matches\t[6]RA Het matches\t[7]AA Hom matches\t[8]RR Hom mismatches\t[9]RA Het mismatches\t[10]AA Hom mismatches\n");
+                printf("# GCiS, Genotype concordance by sample (indels)\n# GCiS\t[2]id\t[3]sample\t[4]non-reference discordance rate\t[5]RR Hom matches\t[6]RA Het matches\t[7]AA Hom matches\t[8]RR Hom mismatches\t[9]RA Het mismatches\t[10]AA Hom mismatches\t[11]dosage r-squared\n");
                 stats = args->smpl_gts_indels;
+                smpl_r_array = args->smpl_r_indels;
             }
             for (i=0; i<args->files->n_smpl; i++)
             {
                 uint64_t m  = stats[i].m[T2S(GT_HET_RA)] + stats[i].m[T2S(GT_HOM_AA)];
                 uint64_t mm = stats[i].mm[T2S(GT_HOM_RR)] + stats[i].mm[T2S(GT_HET_RA)] + stats[i].mm[T2S(GT_HOM_AA)];
+                // Calculate r by formula 19.2 - Biostatistical Analysis 4th edition - Jerrold H. Zar
+                smpl_r_t *smpl_r = smpl_r_array + i;
+                double r = 0.0;
+                if (smpl_r->n) {
+                    double sum_crossprod = smpl_r->xy-(smpl_r->x*smpl_r->y)/smpl_r->n;//per 17.3 machine formula
+                    double x2_xx = smpl_r->x2-(smpl_r->x*smpl_r->x)/smpl_r->n;
+                    double y2_yy = smpl_r->y2-(smpl_r->y*smpl_r->y)/smpl_r->n;
+                    r = (sum_crossprod)/sqrt(x2_xx*y2_yy);
+                }
                 printf("GC%cS\t2\t%s\t%.3f",  x==0 ? 's' : 'i', args->files->samples[i], m+mm ? mm*100.0/(m+mm) : 0);
                 printf("\t%"PRId64"\t%"PRId64"\t%"PRId64"", stats[i].m[T2S(GT_HOM_RR)],stats[i].m[T2S(GT_HET_RA)],stats[i].m[T2S(GT_HOM_AA)]);
-                printf("\t%"PRId64"\t%"PRId64"\t%"PRId64"\n", stats[i].mm[T2S(GT_HOM_RR)],stats[i].mm[T2S(GT_HET_RA)],stats[i].mm[T2S(GT_HOM_AA)]);
+                printf("\t%"PRId64"\t%"PRId64"\t%"PRId64"", stats[i].mm[T2S(GT_HOM_RR)],stats[i].mm[T2S(GT_HET_RA)],stats[i].mm[T2S(GT_HOM_AA)]);
+                if (smpl_r->n && !isnan(r)) printf("\t%f\n", r*r);
+                else printf("\t"NA_STRING"\n");
             }
         }
     }
@@ -1284,19 +1391,22 @@ static void print_stats(args_t *args)
         }
 
 
-        if ( args->exons )
+        printf("# PSI, Per-Sample Indels\n# PSI\t[2]id\t[3]sample\t[4]in-frame\t[5]out-frame\t[6]not applicable\t[7]out/(in+out) ratio\t[8]nHets\t[9]nAA\n");
+        for (id=0; id<args->nstats; id++)
         {
-            printf("# PSI, Per-Sample Indels\n# PSI\t[2]id\t[3]sample\t[4]in-frame\t[5]out-frame\t[6]not applicable\t[7]out/(in+out) ratio\n");
-            for (id=0; id<args->nstats; id++)
+            stats_t *stats = &args->stats[id];
+            for (i=0; i<args->files->n_smpl; i++)
             {
-                stats_t *stats = &args->stats[id];
-                for (i=0; i<args->files->n_smpl; i++)
+                int na = 0, in = 0, out = 0;
+                if ( args->exons )
                 {
-                    int na  = stats->smpl_frm_shifts[i*3 + 0];
-                    int in  = stats->smpl_frm_shifts[i*3 + 1];
-                    int out = stats->smpl_frm_shifts[i*3 + 2];
-                    printf("PSI\t%d\t%s\t%d\t%d\t%d\t%.2f\n", id,args->files->samples[i], in,out,na,in+out?1.0*out/(in+out):0);
+                    na  = stats->smpl_frm_shifts[i*3 + 0];
+                    in  = stats->smpl_frm_shifts[i*3 + 1];
+                    out = stats->smpl_frm_shifts[i*3 + 2];
                 }
+                int nhom = stats->smpl_indel_homs[i];
+                int nhet = stats->smpl_indel_hets[i];
+                printf("PSI\t%d\t%s\t%d\t%d\t%d\t%.2f\t%d\t%d\n", id,args->files->samples[i], in,out,na,in+out?1.0*out/(in+out):0,nhet,nhom);
             }
         }
 
@@ -1355,10 +1465,12 @@ static void usage(void)
     fprintf(stderr, "    -1, --1st-allele-only              include only 1st allele at multiallelic sites\n");
     fprintf(stderr, "    -c, --collapse <string>            treat as identical records with <snps|indels|both|all|some|none>, see man page for details [none]\n");
     fprintf(stderr, "    -d, --depth <int,int,int>          depth distribution: min,max,bin size [0,500,1]\n");
-    fprintf(stderr, "    -e, --exons <file.gz>              tab-delimited file with exons for indel frameshifts (chr,from,to; 1-based, inclusive, bgzip compressed)\n");
+    fprintf(stderr, "    -e, --exclude <expr>               exclude sites for which the expression is true (see man page for details)\n");
+    fprintf(stderr, "    -E, --exons <file.gz>              tab-delimited file with exons for indel frameshifts (chr,from,to; 1-based, inclusive, bgzip compressed)\n");
     fprintf(stderr, "    -f, --apply-filters <list>         require at least one of the listed FILTER strings (e.g. \"PASS,.\")\n");
     fprintf(stderr, "    -F, --fasta-ref <file>             faidx indexed reference sequence file to determine INDEL context\n");
-    fprintf(stderr, "    -i, --split-by-ID                  collect stats for sites with ID separately (known vs novel)\n");
+    fprintf(stderr, "    -i, --include <expr>               select sites for which the expression is true (see man page for details)\n");
+    fprintf(stderr, "    -I, --split-by-ID                  collect stats for sites with ID separately (known vs novel)\n");
     fprintf(stderr, "    -r, --regions <region>             restrict to comma-separated list of regions\n");
     fprintf(stderr, "    -R, --regions-file <file>          restrict to regions listed in a file\n");
     fprintf(stderr, "    -s, --samples <list>               list of samples for sample stats, \"-\" to include all samples\n");
@@ -1383,6 +1495,8 @@ int main_vcfstats(int argc, char *argv[])
     static struct option loptions[] =
     {
         {"1st-allele-only",0,0,'1'},
+        {"include",1,0,'i'},
+        {"exclude",1,0,'e'},
         {"help",0,0,'h'},
         {"collapse",1,0,'c'},
         {"regions",1,0,'r'},
@@ -1390,17 +1504,17 @@ int main_vcfstats(int argc, char *argv[])
         {"verbose",0,0,'v'},
         {"depth",1,0,'d'},
         {"apply-filters",1,0,'f'},
-        {"exons",1,0,'e'},
+        {"exons",1,0,'E'},
         {"samples",1,0,'s'},
         {"samples-file",1,0,'S'},
-        {"split-by-ID",0,0,'i'},
+        {"split-by-ID",0,0,'I'},
         {"targets",1,0,'t'},
         {"targets-file",1,0,'T'},
         {"fasta-ref",1,0,'F'},
         {"user-tstv",1,0,'u'},
         {0,0,0,0}
     };
-    while ((c = getopt_long(argc, argv, "hc:r:R:e:s:S:d:it:T:F:f:1u:v",loptions,NULL)) >= 0) {
+    while ((c = getopt_long(argc, argv, "hc:r:R:e:s:S:d:i:t:T:F:f:1u:vIE:",loptions,NULL)) >= 0) {
         switch (c) {
             case 'u': add_user_stats(args,optarg); break;
             case '1': args->first_allele_only = 1; break;
@@ -1427,10 +1541,12 @@ int main_vcfstats(int argc, char *argv[])
             case 'f': args->files->apply_filters = optarg; break;
             case 'r': args->regions_list = optarg; break;
             case 'R': args->regions_list = optarg; regions_is_file = 1; break;
-            case 'e': args->exons_fname = optarg; break;
+            case 'E': args->exons_fname = optarg; break;
             case 's': args->samples_list = optarg; break;
             case 'S': args->samples_list = optarg; args->samples_is_file = 1; break;
-            case 'i': args->split_by_id = 1; break;
+            case 'I': args->split_by_id = 1; break;
+            case 'e': args->filter_str = optarg; args->filter_logic |= FLT_EXCLUDE; break;
+            case 'i': args->filter_str = optarg; args->filter_logic |= FLT_INCLUDE; break;
             case 'h':
             case '?': usage();
             default: error("Unknown argument: %s\n", optarg);
